@@ -3,8 +3,7 @@ require 'spec_helper'
 describe 'horizon' do
 
   let :params do
-    { 'secret_key' => 'elj1IWiLoWHgcyYxFVLj7cM5rGOOxWl0',
-      'fqdn'       => '*' }
+    { 'secret_key' => 'elj1IWiLoWHgcyYxFVLj7cM5rGOOxWl0' }
   end
 
   let :pre_condition do
@@ -57,7 +56,7 @@ describe 'horizon' do
           "LOGIN_URL = '#{platforms_params[:root_url]}/auth/login/'",
           "LOGOUT_URL = '#{platforms_params[:root_url]}/auth/logout/'",
           "LOGIN_REDIRECT_URL = '#{platforms_params[:root_url]}/'",
-          "ALLOWED_HOSTS = ['*', ]",
+          "ALLOWED_HOSTS = ['some.host.tld', ]",
           "  'identity': 3,",
           'HORIZON_CONFIG["password_autocomplete"] = "off"',
           'HORIZON_CONFIG["images_panel"] = "legacy"',
@@ -76,7 +75,10 @@ describe 'horizon' do
           'API_RESULT_LIMIT = 1000',
           'TIME_ZONE = "UTC"',
           'COMPRESS_OFFLINE = True',
-          "FILE_UPLOAD_TEMP_DIR = '/tmp'"
+          "FILE_UPLOAD_TEMP_DIR = '/tmp'",
+          "OPENSTACK_HEAT_STACK = {",
+          "    'enable_user_pass': True",
+          "}",
         ])
 
         # From internals of verify_contents, get the contents to check for absence of a line
@@ -92,48 +94,53 @@ describe 'horizon' do
     context 'with overridden parameters' do
       before do
         params.merge!({
-          :cache_backend                => 'horizon.backends.memcached.HorizonMemcached',
-          :cache_options                => {'SOCKET_TIMEOUT' => 1,'SERVER_RETRIES' => 1,'DEAD_RETRY' => 1},
-          :cache_server_ip              => '10.0.0.1',
-          :django_session_engine        => 'django.contrib.sessions.backends.cache',
-          :keystone_default_role        => 'SwiftOperator',
-          :keystone_url                 => 'https://keystone.example.com:4682',
-          :ssl_no_verify                => true,
-          :log_handler                  => 'syslog',
-          :log_level                    => 'DEBUG',
-          :openstack_endpoint_type      => 'internalURL',
-          :secondary_endpoint_type      => 'ANY-VALUE',
-          :django_debug                 => true,
-          :api_result_limit             => 4682,
-          :compress_offline             => false,
-          :hypervisor_options           => {'can_set_mount_point' => false, 'can_set_password' => true },
-          :cinder_options               => {'enable_backup' => true },
-          :keystone_options             => {'name' => 'native', 'can_edit_user' => true, 'can_edit_group' => true, 'can_edit_project' => true, 'can_edit_domain' => false, 'can_edit_role' => false},
-          :neutron_options              => {'enable_lb' => true, 'enable_firewall' => true, 'enable_quotas' => false, 'enable_security_group' => false, 'enable_vpn' => true,
+          :cache_backend                  => 'horizon.backends.memcached.HorizonMemcached',
+          :cache_options                  => {'SOCKET_TIMEOUT' => 1,'SERVER_RETRIES' => 1,'DEAD_RETRY' => 1},
+          :cache_server_ip                => '10.0.0.1',
+          :django_session_engine          => 'django.contrib.sessions.backends.cache',
+          :keystone_default_role          => 'SwiftOperator',
+          :keystone_url                   => 'https://keystone.example.com:4682',
+          :ssl_no_verify                  => true,
+          :log_handler                    => 'syslog',
+          :log_level                      => 'DEBUG',
+          :openstack_endpoint_type        => 'internalURL',
+          :secondary_endpoint_type        => 'ANY-VALUE',
+          :django_debug                   => true,
+          :api_result_limit               => 4682,
+          :compress_offline               => false,
+          :hypervisor_options             => {'can_set_mount_point' => false, 'can_set_password' => true },
+          :cinder_options                 => {'enable_backup' => true },
+          :keystone_options               => {'name' => 'native', 'can_edit_user' => true, 'can_edit_group' => true, 'can_edit_project' => true, 'can_edit_domain' => false, 'can_edit_role' => false},
+          :neutron_options                => {'enable_lb' => true, 'enable_firewall' => true, 'enable_quotas' => false, 'enable_security_group' => false, 'enable_vpn' => true,
                                             'enable_distributed_router' => false, 'enable_ha_router' => false, 'profile_support' => 'cisco',
                                             'supported_provider_types' => ['flat', 'vxlan'], 'supported_vnic_types' => ['*'], 'default_ipv4_subnet_pool_label' => 'None', },
-          :file_upload_temp_dir         => '/var/spool/horizon',
-          :secure_cookies               => true,
-          :api_versions                 => {'identity' => 2.0},
-          :keystone_multidomain_support => true,
-          :keystone_default_domain      => 'domain.tld',
-          :overview_days_range          => 1,
-          :session_timeout              => 1800,
-          :timezone                     => 'Asia/Shanghai',
-          :available_themes             => [
+          :instance_options               => {'disable_image' => true, 'disable_instance_snapshot' => true, 'disable_volume' => true, 'disable_volume_snapshot' => true, 'create_volume' => false },
+          :file_upload_temp_dir           => '/var/spool/horizon',
+          :secure_cookies                 => true,
+          :api_versions                   => {'identity' => 2.0},
+          :keystone_multidomain_support   => true,
+          :keystone_default_domain        => 'domain.tld',
+          :overview_days_range            => 1,
+          :session_timeout                => 1800,
+          :timezone                       => 'Asia/Shanghai',
+          :available_themes               => [
             { 'name' => 'default', 'label' => 'Default', 'path' => 'themes/default' },
             { 'name' => 'material', 'label' => 'Material', 'path' => 'themes/material' },
           ],
-          :default_theme                => 'default',
-          :password_autocomplete        => 'on',
-          :images_panel                 => 'angular',
+          :default_theme                  => 'default',
+          :password_autocomplete          => 'on',
+          :images_panel                   => 'angular',
+          :create_image_defaults          => {'image_visibility' => 'private'},
+          :password_retrieve              => true,
+          :enable_secure_proxy_ssl_header => true,
         })
       end
 
       it 'generates local_settings.py' do
         verify_concat_fragment_contents(catalogue, 'local_settings.py', [
           'DEBUG = True',
-          "ALLOWED_HOSTS = ['*', ]",
+          "ALLOWED_HOSTS = ['some.host.tld', ]",
+          "SECURE_PROXY_SSL_HEADER = ('HTTP_X_FORWARDED_PROTO', 'https')",
           'CSRF_COOKIE_SECURE = True',
           'SESSION_COOKIE_SECURE = True',
           "  'identity': 2.0,",
@@ -171,6 +178,16 @@ describe 'horizon' do
           "    'profile_support': 'cisco',",
           "    'supported_provider_types': ['flat', 'vxlan'],",
           "    'supported_vnic_types': ['*'],",
+          'OPENSTACK_ENABLE_PASSWORD_RETRIEVE = True',
+          'CREATE_IMAGE_DEFAULTS = {',
+          "    'image_visibility': 'private',",
+          "    'config_drive': False,",
+          "    'create_volume': False,",
+          "    'disable_image': True,",
+          "    'disable_instance_snapshot': True,",
+          "    'disable_volume': True,",
+          "    'disable_volume_snapshot': True,",
+          "    'enable_scheduler_hints': True,",
           'OPENSTACK_ENDPOINT_TYPE = "internalURL"',
           'SECONDARY_ENDPOINT_TYPE = "ANY-VALUE"',
           'API_RESULT_LIMIT = 4682',
@@ -195,6 +212,19 @@ describe 'horizon' do
       it { is_expected.to contain_file(params[:file_upload_temp_dir]) }
     end
 
+    context 'with enable_user_pass disabled' do
+      before do
+        params.merge!({ :enable_user_pass => false })
+      end
+      it {
+        verify_concat_fragment_contents(catalogue, 'local_settings.py', [
+          "OPENSTACK_HEAT_STACK = {",
+          "    'enable_user_pass': False",
+          "}",
+        ])
+      }
+    end
+
     context 'with overridden parameters and cache_server_ip array' do
       before do
         params.merge!({
@@ -221,8 +251,8 @@ describe 'horizon' do
 
       it {
         is_expected.to contain_package('python-memcache').with(
-          :ensure => 'present',
-          :tag    => ['openstack']
+          :tag    => ['openstack', 'horizon-package'],
+          :name   => platforms_params[:memcache_package],
          )
       }
     end
@@ -243,7 +273,6 @@ describe 'horizon' do
       end
     end
 
-
     context 'with ssl enabled' do
       before do
         params.merge!({
@@ -266,6 +295,44 @@ describe 'horizon' do
       end
     end
 
+    context 'with overridden http and https ports' do
+      before do
+        params.merge!({
+          :http_port  => 1028,
+          :https_port => 1029,
+        })
+      end
+
+      it 'configures apache' do
+        is_expected.to contain_class('horizon::wsgi::apache').with({
+          :http_port  => 1028,
+          :https_port => 1029,
+        })
+      end
+    end
+
+    context 'with default root_path' do
+      it 'configures apache' do
+        is_expected.to contain_class('horizon::wsgi::apache').with({
+          :root_path    => "#{platforms_params[:root_path]}",
+        })
+      end
+    end
+
+    context 'with root_path set to /tmp/horizon' do
+      before do
+        params.merge!({
+          :root_path    => '/tmp/horizon',
+        })
+      end
+
+      it 'configures apache' do
+        is_expected.to contain_class('horizon::wsgi::apache').with({
+          :root_path    => '/tmp/horizon',
+        })
+      end
+    end
+
     context 'without apache' do
       before do
         params.merge!({ :configure_apache => false })
@@ -411,6 +478,156 @@ describe 'horizon' do
         ])
       end
     end
+
+    context 'with disable password reveal enabled' do
+      before do
+        params.merge!({
+          :disable_password_reveal => true
+        })
+      end
+
+      it 'disable_password_reveal is configured' do
+        verify_concat_fragment_contents(catalogue, 'local_settings.py', [
+          'HORIZON_CONFIG["disable_password_reveal"] = True',
+        ])
+      end
+    end
+
+    context 'with enforce password check enabled' do
+      before do
+        params.merge!({
+          :enforce_password_check => true
+        })
+      end
+
+      it 'enforce_password_check is configured' do
+        verify_concat_fragment_contents(catalogue, 'local_settings.py', [
+          'HORIZON_CONFIG["enforce_password_check"] = True',
+        ])
+      end
+    end
+
+    context 'with disallow iframe embed enabled' do
+      before do
+        params.merge!({
+                        :disallow_iframe_embed => true
+                      })
+      end
+
+      it 'disallow_iframe_embed is configured' do
+        verify_concat_fragment_contents(catalogue, 'local_settings.py', [
+                                          'HORIZON_CONFIG["disallow_iframe_embed"] = True',
+                                        ])
+      end
+    end
+
+    context 'with websso enabled' do
+      before do
+        params.merge!({
+                        :websso_enabled => 'True',
+                        :websso_initial_choice => 'acme',
+                        :websso_choices => [
+                          ['oidc', 'OpenID Connect'],
+                          ['saml2', 'Security Assertion Markup Language'],
+                        ],
+                        :websso_idp_mapping => {
+                          'acme_oidc'  => ['acme', 'oidc'],
+                          'acme_saml2' => ['acme', 'saml2'],
+                        }
+                      })
+      end
+      it 'configures websso options' do
+        verify_concat_fragment_contents(catalogue, 'local_settings.py', [
+                                          'WEBSSO_ENABLED = True',
+                                          'WEBSSO_INITIAL_CHOICE = "acme"',
+                                          'WEBSSO_CHOICES = (',
+                                          '    ("credentials", _("Keystone Credentials")),',
+                                          '    ("oidc", _("OpenID Connect")),',
+                                          '    ("saml2", _("Security Assertion Markup Language")),',
+                                          ')',
+                                          'WEBSSO_IDP_MAPPING = {',
+                                          '    "acme_oidc": ("acme", "oidc"),',
+                                          '    "acme_saml2": ("acme", "saml2"),',
+                                          '}',
+                                        ])
+      end
+    end
+
+    context 'with customization_module provided' do
+      before do
+        params.merge!({
+          :help_url                => 'https://docs.openstack.org',
+          :customization_module    => 'my_project.overrides',
+          :local_settings_template => fixtures_path + '/override_local_settings.py.erb'
+        })
+      end
+
+      it 'uses the custom local_settings.py template' do
+        verify_concat_fragment_contents(catalogue, 'local_settings.py', [
+          '# Custom local_settings.py',
+          "HORIZON_CONFIG = {",
+          "    'dashboards': ('project', 'admin', 'settings',),",
+          "    'default_dashboard': 'project',",
+          "    'user_home': 'openstack_dashboard.views.get_user_home',",
+          "    'ajax_queue_limit': 10,",
+          "    'auto_fade_alerts': {",
+          "        'delay': 3000,",
+          "        'fade_duration': 1500,",
+          "        'types': ['alert-success', 'alert-info']",
+          "    },",
+          "    'help_url': \"https://docs.openstack.org\",",
+          "    'exceptions': {'recoverable': exceptions.RECOVERABLE,",
+          "                   'not_found': exceptions.NOT_FOUND,",
+          "                   'unauthorized': exceptions.UNAUTHORIZED},",
+          "    'customization_module': 'my_project.overrides',",
+          "}",
+        ])
+      end
+    end
+
+    context 'with customization_module empty' do
+      before do
+        params.merge!({
+          :help_url                => 'https://docs.openstack.org',
+          :customization_module    => '',
+          :local_settings_template => fixtures_path + '/override_local_settings.py.erb'
+        })
+      end
+
+      it 'uses the custom local_settings.py template' do
+        verify_concat_fragment_contents(catalogue, 'local_settings.py', [
+          '# Custom local_settings.py',
+          "HORIZON_CONFIG = {",
+          "    'dashboards': ('project', 'admin', 'settings',),",
+          "    'default_dashboard': 'project',",
+          "    'user_home': 'openstack_dashboard.views.get_user_home',",
+          "    'ajax_queue_limit': 10,",
+          "    'auto_fade_alerts': {",
+          "        'delay': 3000,",
+          "        'fade_duration': 1500,",
+          "        'types': ['alert-success', 'alert-info']",
+          "    },",
+          "    'help_url': \"https://docs.openstack.org\",",
+          "    'exceptions': {'recoverable': exceptions.RECOVERABLE,",
+          "                   'not_found': exceptions.NOT_FOUND,",
+          "                   'unauthorized': exceptions.UNAUTHORIZED},",
+          "}",
+        ])
+      end
+    end
+
+    context 'with upload mode' do
+      before do
+        params.merge!({
+          :horizon_upload_mode  => 'direct',
+        })
+      end
+      it 'sets HORIZON_IMAGES_UPLOAD_MODE in local_settings.py' do
+        verify_concat_fragment_contents(catalogue, 'local_settings.py', [
+	  'HORIZON_IMAGES_UPLOAD_MODE = direct',
+        ])
+      end
+    end
   end
 
   shared_examples_for 'horizon on RedHat' do
@@ -436,7 +653,6 @@ describe 'horizon' do
       let (:facts) do
         facts.merge!(OSDefaults.get_facts({
           :fqdn           => 'some.host.tld',
-          :processorcount => 2,
           :concat_basedir => '/var/lib/puppet/concat'
         }))
       end
@@ -444,13 +660,28 @@ describe 'horizon' do
       let(:platforms_params) do
         case facts[:osfamily]
         when 'Debian'
-          { :config_file       => '/etc/openstack-dashboard/local_settings.py',
-            :package_name      => 'openstack-dashboard',
-            :root_url          => '/horizon' }
+          if facts[:os_package_type] == 'debian'
+            { :config_file      => '/etc/openstack-dashboard/local_settings.py',
+              :package_name     => 'openstack-dashboard-apache',
+              :root_url         => '/horizon',
+              :root_path        => '/var/lib/openstack-dashboard',
+              :memcache_package => 'python3-memcache',
+            }
+          else
+            { :config_file      => '/etc/openstack-dashboard/local_settings.py',
+              :package_name     => 'openstack-dashboard',
+              :root_url         => '/horizon',
+              :root_path        => '/var/lib/openstack-dashboard',
+              :memcache_package => 'python-memcache',
+            }
+          end
         when 'RedHat'
-          { :config_file       => '/etc/openstack-dashboard/local_settings',
-            :package_name      => 'openstack-dashboard',
-            :root_url          => '/dashboard' }
+          { :config_file      => '/etc/openstack-dashboard/local_settings',
+            :package_name     => 'openstack-dashboard',
+            :root_url         => '/dashboard',
+            :root_path        => '/usr/share/openstack-dashboard',
+            :memcache_package => 'python-memcached',
+          }
         end
       end
 
@@ -458,5 +689,4 @@ describe 'horizon' do
       it_behaves_like "horizon on #{facts[:osfamily]}"
     end
   end
-
 end
