@@ -3,7 +3,7 @@ require 'spec_helper'
 describe 'horizon::wsgi::apache' do
 
   let :params do
-    { :fqdn           => '*',
+    {
       :servername     => 'some.host.tld',
       :wsgi_processes => '3',
       :wsgi_threads   => '10',
@@ -31,26 +31,27 @@ describe 'horizon::wsgi::apache' do
         is_expected.to contain_file(platforms_params[:httpd_config_file])
         is_expected.to contain_package('horizon').with_ensure('present')
         is_expected.to contain_apache__vhost('horizon_vhost').with(
-          'servername'           => 'some.host.tld',
-          'access_log_file'      => 'horizon_access.log',
-          'error_log_file'       => 'horizon_error.log',
-          'priority'             => '15',
-          'serveraliases'        => ['*'],
-          'docroot'              => '/var/www/',
-          'ssl'                  => 'false',
-          'port'                 => '80',
-          'redirectmatch_status' => 'permanent',
-          'redirectmatch_regexp' => '^/$',
-          'redirectmatch_dest'   => platforms_params[:root_url],
-          'wsgi_script_aliases'  => { platforms_params[:root_url] => '/usr/share/openstack-dashboard/openstack_dashboard/wsgi/django.wsgi' },
-          'wsgi_process_group'   => platforms_params[:wsgi_group],
-          'wsgi_daemon_process'  => platforms_params[:wsgi_group],
-          'wsgi_daemon_process_options' => { 'processes' => params[:wsgi_processes], 'threads' => params[:wsgi_threads], 'user' => platforms_params[:unix_user], 'group' => platforms_params[:unix_group] }
+          'servername'             => 'some.host.tld',
+          'access_log_file'        => 'horizon_access.log',
+          'error_log_file'         => 'horizon_error.log',
+          'priority'               => '15',
+          'serveraliases'          => ['some.host.tld'],
+          'docroot'                => '/var/www/',
+          'ssl'                    => 'false',
+          'port'                   => '80',
+          'redirectmatch_status'   => 'permanent',
+          'redirectmatch_regexp'   => '^/$',
+          'redirectmatch_dest'     => platforms_params[:root_url],
+          'wsgi_script_aliases'    => { platforms_params[:root_url] => '/usr/share/openstack-dashboard/openstack_dashboard/wsgi/django.wsgi' },
+          'wsgi_process_group'     => platforms_params[:wsgi_group],
+          'wsgi_daemon_process'    => platforms_params[:wsgi_group],
+          'wsgi_application_group' => '%{GLOBAL}',
+          'wsgi_daemon_process_options' => { 'processes' => params[:wsgi_processes], 'threads' => params[:wsgi_threads], 'user' => platforms_params[:unix_user], 'group' => platforms_params[:unix_group], 'display-name' => 'horizon' }
          )
       end
     end
 
-    context 'with overriden parameters' do
+    context 'with overridden parameters' do
       before do
         params.merge!({
           :priority      => '10',
@@ -67,25 +68,49 @@ describe 'horizon::wsgi::apache' do
         is_expected.to contain_file(platforms_params[:httpd_config_file])
         is_expected.to contain_package('horizon').with_ensure('present')
         is_expected.to contain_apache__vhost('horizon_vhost').with(
-          'servername'           => 'some.host.tld',
-          'access_log_file'      => 'horizon_access.log',
-          'error_log_file'       => 'horizon_error.log',
-          'priority'             => params[:priority],
-          'serveraliases'        => ['*'],
-          'docroot'              => '/var/www/',
-          'ssl'                  => 'false',
-          'port'                 => '80',
-          'redirectmatch_status' => 'temp',
-          'redirectmatch_regexp' => '^/$',
-          'redirectmatch_dest'   => platforms_params[:root_url],
-          'wsgi_script_aliases'  => { platforms_params[:root_url] => '/usr/share/openstack-dashboard/openstack_dashboard/wsgi/django.wsgi' },
-          'wsgi_process_group'   => platforms_params[:wsgi_group],
-          'wsgi_daemon_process'  => platforms_params[:wsgi_group],
-          'wsgi_daemon_process_options' => { 'processes' => params[:wsgi_processes], 'threads' => params[:wsgi_threads], 'user' => platforms_params[:unix_user], 'group' => platforms_params[:unix_group] }
+          'servername'             => 'some.host.tld',
+          'access_log_file'        => 'horizon_access.log',
+          'error_log_file'         => 'horizon_error.log',
+          'priority'               => params[:priority],
+          'serveraliases'          => ['some.host.tld'],
+          'docroot'                => '/var/www/',
+          'ssl'                    => 'false',
+          'port'                   => '80',
+          'redirectmatch_status'   => 'temp',
+          'redirectmatch_regexp'   => '^/$',
+          'redirectmatch_dest'     => platforms_params[:root_url],
+          'wsgi_script_aliases'    => { platforms_params[:root_url] => '/usr/share/openstack-dashboard/openstack_dashboard/wsgi/django.wsgi' },
+          'wsgi_process_group'     => platforms_params[:wsgi_group],
+          'wsgi_daemon_process'    => platforms_params[:wsgi_group],
+          'wsgi_application_group' => '%{GLOBAL}',
+          'wsgi_daemon_process_options' => { 'processes' => params[:wsgi_processes], 'threads' => params[:wsgi_threads], 'user' => platforms_params[:unix_user], 'group' => platforms_params[:unix_group], 'display-name' => 'horizon' }
          )
       end
     end
 
+    context 'with custom_custom_wsgi_options' do
+      before do
+        params.merge!({
+          :custom_wsgi_process_options => {
+            'user'        => 'myuser',
+            'python_path' => '/my/python/admin/path',
+          },
+        })
+      end
+      it 'configures apache' do
+        is_expected.to contain_apache__vhost('horizon_vhost').with(
+          'wsgi_daemon_process_options' => {
+            'processes'    => params[:wsgi_processes],
+            'threads'      => params[:wsgi_threads],
+            'user'         => 'myuser',
+            'group'        => platforms_params[:unix_group],
+            'display-name' => 'horizon',
+            'python_path'  => '/my/python/admin/path'
+          }
+        )
+      end
+    end
+
     context 'with ssl enabled' do
       before do
         params.merge!({
@@ -106,7 +131,7 @@ describe 'horizon::wsgi::apache' do
           'access_log_file'        => 'horizon_ssl_access.log',
           'error_log_file'         => 'horizon_ssl_error.log',
           'priority'               => '15',
-          'serveraliases'          => ['*'],
+          'serveraliases'          => ['some.host.tld'],
           'docroot'                => '/var/www/',
           'ssl'                    => 'true',
           'port'                   => '443',
@@ -118,34 +143,31 @@ describe 'horizon::wsgi::apache' do
           'redirectmatch_dest'     => platforms_params[:root_url],
           'wsgi_process_group'     => 'horizon-ssl',
           'wsgi_daemon_process'    => 'horizon-ssl',
+          'wsgi_application_group' => '%{GLOBAL}',
           'wsgi_script_aliases'  => { platforms_params[:root_url] => '/usr/share/openstack-dashboard/openstack_dashboard/wsgi/django.wsgi' },
         )}
 
         it { is_expected.to contain_apache__vhost('horizon_vhost').with(
-          'servername'           => 'some.host.tld',
-          'access_log_file'      => 'horizon_access.log',
-          'error_log_file'       => 'horizon_error.log',
-          'priority'             => '15',
-          'serveraliases'        => ['*'],
-          'docroot'              => '/var/www/',
-          'ssl'                  => 'false',
-          'port'                 => '80',
-          'redirectmatch_status' => 'permanent',
-          'redirectmatch_regexp' => '(.*)',
-          'redirectmatch_dest'   => 'https://some.host.tld',
-          'wsgi_process_group'   => platforms_params[:wsgi_group],
-          'wsgi_daemon_process'  => platforms_params[:wsgi_group],
-          'wsgi_script_aliases'  => { platforms_params[:root_url] => '/usr/share/openstack-dashboard/openstack_dashboard/wsgi/django.wsgi' },
+          'servername'             => 'some.host.tld',
+          'access_log_file'        => 'horizon_access.log',
+          'error_log_file'         => 'horizon_error.log',
+          'priority'               => '15',
+          'serveraliases'          => ['some.host.tld'],
+          'docroot'                => '/var/www/',
+          'ssl'                    => 'false',
+          'port'                   => '80',
+          'redirectmatch_status'   => 'permanent',
+          'redirectmatch_regexp'   => '(.*)',
+          'redirectmatch_dest'     => 'https://some.host.tld',
+          'wsgi_process_group'     => platforms_params[:wsgi_group],
+          'wsgi_daemon_process'    => platforms_params[:wsgi_group],
+          'wsgi_application_group' => '%{GLOBAL}',
+          'wsgi_script_aliases'    => { platforms_params[:root_url] => '/usr/share/openstack-dashboard/openstack_dashboard/wsgi/django.wsgi' },
         )}
       end
 
       context 'without required parameters' do
 
-        context 'without horizon_ca parameter' do
-          before { params.delete(:horizon_ca) }
-          it_raises 'a Puppet::Error', /The horizon_ca parameter is required when listen_ssl is true/
-        end
-
         context 'without horizon_cert parameter' do
           before { params.delete(:horizon_cert) }
           it_raises 'a Puppet::Error', /The horizon_cert parameter is required when listen_ssl is true/
@@ -180,9 +202,17 @@ describe 'horizon::wsgi::apache' do
         before do
           params.merge!({
             :root_url => '/',
+            :root_path => '/tmp/horizon'
           })
         end
 
+        it 'configures apache with correct root url' do
+          is_expected.to contain_apache__vhost('horizon_vhost').with(
+            'aliases'             => [{'alias' => '/static', 'path' => '/tmp/horizon/static'}],
+            'wsgi_script_aliases' => { '/' => '/usr/share/openstack-dashboard/openstack_dashboard/wsgi/django.wsgi' },
+          )
+        end
+
         it 'should not configure redirectmatch' do
           is_expected.to_not contain_apache__vhost('horizon_vhost').with(
             'redirectmatch_regexp' => '^/$',
@@ -195,6 +225,24 @@ describe 'horizon::wsgi::apache' do
         end
       end
 
+      context 'with root_url set to empty' do
+        before do
+          params.merge!({
+            :root_url => '',
+          })
+        end
+
+        it 'should not configure redirectmatch' do
+          is_expected.to_not contain_apache__vhost('horizon_vhost').with(
+            'redirectmatch_regexp' => '(.*)',
+            'redirectmatch_dest'   => '/'
+          )
+          is_expected.to_not contain_apache__vhost('horizon_ssl_vhost').with(
+            'redirectmatch_regexp' => '^/$',
+            'redirectmatch_dest'   => ''
+          )
+        end
+      end
 
     end
   end
@@ -214,20 +262,60 @@ describe 'horizon::wsgi::apache' do
         )
       end
     end
+
+    context 'with root_path set to /tmp/horizon' do
+      before do
+        params.merge!({
+          :root_path => '/tmp/horizon',
+        })
+      end
+
+      it 'configures webroot alias' do
+        if (Gem::Version.new(Puppet.version) >= Gem::Version.new('4.0'))
+          is_expected.to contain_apache__vhost('horizon_vhost').with(
+            'aliases' => [{'alias' => '/dashboard/static', 'path' => '/tmp/horizon/static'}],
+          )
+        else
+          is_expected.to contain_apache__vhost('horizon_vhost').with(
+            'aliases' => [['alias', '/dashboard/static'], ['path', '/tmp/horizon/static']],
+          )
+        end
+      end
+    end
   end
 
   shared_examples_for 'apache for horizon on Debian platforms' do
     it 'configures webroot alias' do
       if (Gem::Version.new(Puppet.version) >= Gem::Version.new('4.0'))
         is_expected.to contain_apache__vhost('horizon_vhost').with(
-          'aliases' => [{'alias' => '/horizon/static', 'path' => '/usr/share/openstack-dashboard/static'}],
+          'aliases' => [{'alias' => '/horizon/static', 'path' => '/var/lib/openstack-dashboard/static'}],
         )
       else
         is_expected.to contain_apache__vhost('horizon_vhost').with(
-          'aliases' => [['alias', '/horizon/static'], ['path', '/usr/share/openstack-dashboard/static']],
+          'aliases' => [['alias', '/horizon/static'], ['path', '/var/lib/openstack-dashboard/static']],
         )
       end
     end
+
+    context 'with root_path set to /tmp/horizon' do
+      before do
+        params.merge!({
+          :root_path => '/tmp/horizon',
+        })
+      end
+
+      it 'configures webroot alias' do
+        if (Gem::Version.new(Puppet.version) >= Gem::Version.new('4.0'))
+          is_expected.to contain_apache__vhost('horizon_vhost').with(
+            'aliases' => [{'alias' => '/horizon/static', 'path' => '/tmp/horizon/static'}],
+          )
+        else
+          is_expected.to contain_apache__vhost('horizon_vhost').with(
+            'aliases' => [['alias', '/horizon/static'], ['path', '/tmp/horizon/static']],
+          )
+        end
+      end
+    end
   end
 
 
@@ -242,7 +330,6 @@ describe 'horizon::wsgi::apache' do
 
         facts.merge!(OSDefaults.get_facts({
           :fqdn           => 'some.host.tld',
-          :processorcount => 2,
           :concat_basedir => '/var/lib/puppet/concat'
         }))
       end
